



#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <signal.h>


#include <coda_file.h>
#include <coda_et.h>
#include <coda_parse.h>
#include <cern_c.h>

int quit=0;
int NUM_EVENTS_READ=0;
int NUM_PHYSICS_EVENTS=0;
int NUM_EVENTS_TO_PROCESS=0;

int EVENT_NUMBER;
int RESET_HISTOS=0;



int read_event(FILE *file,CODA_header_t **evt);
void ParseCommandLineArgs(int argc,char *argv[]);
void Usage(void);
void FindXY_hits(void);
int hist_book(void);
void hist_end(void);
int ReadConfiguration(int run_number);
void ctrlCHandle(int x);

/*------------ PAW DEFINES ----------------- */
#define MEMH 8000000
#define LREC 1024 /* record length of hbook direct access file in WORDS */
#define LUN 3 /* logical unit number of hbook file */
/* declare the hbook common */
float pawc_[MEMH];
float hcfitd_[25];
int quest_[100];
FILE *file;
char SESSION[64]="";
char input_filename[256]="";
char output_filename[256]="";
char GLOBAL_SECTION[256]="";

int VERBOSE=0;

#define MAX_ROC_BANKS  16
#define MAX_THITS     256

#define ADC_SLOT	7
#define TDC_SLOT_1	6
#define TDC_SLOT_2	5


typedef struct {
	TDC1877_t x,y;
}XY_hit_t;

int N_TDC_HITS;
TDC1877_t TDC[MAX_THITS];
TDC1877_t XHITS[MAX_THITS];
TDC1877_t YHITS[MAX_THITS];
XY_hit_t XYHITS[MAX_THITS];
int N_XHITS,N_YHITS,N_XYHITS;

unsigned int PEDESTAL[64];

/* Routine declarations */
void ParseCommandLineArgs(int argc,char *argv[]);
void Usage(void);
int hist_book(void);
void hist_fill(unsigned int adc[64], TDC1877_t *tdc, int n_tdc_hits);
void hist_end(void);
void ctrlCHandle(int x);
void SIGHUPHandle(int x);




int main(int argc,char *argv[])
{
	CODA_roc_data_t *roc_data[MAX_ROC_BANKS];
	int Nroc_ptrs;
	CODA_event_id_t *event_id;
	CODA_header_t *evt;
	TDC1877_t tdc[MAX_THITS];
	unsigned int adc[64];
	unsigned int LRS1881M_slots, LRS1877_slots;
	int n_tdc_hits;
	int err;
	int i,j;

	/* Catch signals */
	signal(SIGINT,ctrlCHandle);
	signal(SIGHUP,SIGHUPHandle);

	ParseCommandLineArgs(argc,argv);

	ReadConfiguration(0);
	hist_book();

	/* Open the event source (file or ET) */
	if(strlen(SESSION)){
		if(init_et(SESSION)){
			fprintf(stderr,"unable to connect to ET (session=\"%s\") !\n\n",SESSION);
			return -1;
		}
	}else{
		/* Open the input file. Assume NO header so first byte starts CODA data */
		file=OpenCODAFile(input_filename);
		if(!file){
			fprintf(stderr,"unable to open file \"%s\" for reading!\n\n",input_filename);
			return -1;
		}
	}

	printf("\nReset histograms with:\n\tkill -HUP %d\n\n",getpid());

	/* Loop over events */
	printf("Entering event loop ...\n");
	while(!quit){

		if(VERBOSE)printf("\n");

		/* read in next event */
		if(strlen(SESSION)){
			if(ReadEventFromET(&evt)){
				/* problem reading the event. Don't die! wait for ET system to come back to life */
				primex_et_wait_for_alive();
				continue;
			}
		}else{
			if(ReadCODAEventFromFile(file,&evt))break;
		}

		/* Advance and check event counter */
		NUM_EVENTS_READ++;
		if((NUM_EVENTS_READ>=NUM_EVENTS_TO_PROCESS) && (NUM_EVENTS_TO_PROCESS!=0))break;

		/* Get pointers to the data read from the ROCs */
		Nroc_ptrs = MAX_ROC_BANKS;
		if(GetBankPtrs((char*)evt, &event_id, roc_data, &Nroc_ptrs ))continue;
		if(VERBOSE)printf("\tNroc_ptrs=%d\n",Nroc_ptrs);
		NUM_PHYSICS_EVENTS++;

		/* Loop over ROCS, deciphering the 1881M and 1877 data */
		LRS1881M_slots = 0x01<<ADC_SLOT;
		LRS1877_slots  = (0x01<<TDC_SLOT_1) | (0x01<<TDC_SLOT_2);
		for(i=0;i<Nroc_ptrs;i++){

			/* Update screen */
			if(NUM_EVENTS_READ%100 == 0)printf("   %d events processed\r",NUM_EVENTS_READ);fflush(stdout);

			n_tdc_hits = MAX_THITS;	
			err = ParseAlldata(roc_data[i],LRS1881M_slots ,adc ,LRS1877_slots ,tdc ,&n_tdc_hits);
			if(err)continue;
			for(i=0;i<64;i++)adc[i] -= (adc[i]>PEDESTAL[i] ? PEDESTAL[i]:adc[i]);

			/* print debugging info if requested */
			if(VERBOSE){
				int i;

				printf("ADC:\n");
				for(i=0;i<64;i++)if(adc[i])printf("\tadc[%d]=%d\n",i,adc[i]);
				printf("\n");

				printf("TDC:\n");
				for(i=0;i<n_tdc_hits;i++){
					printf("tdc[%d]=%d  edge=%s slot=%d\n"
						,tdc[i].id
						,tdc[i].val
						,tdc[i].edge==LRS1877_FALLING_EDGE ? "falling":"rising"
						,tdc[i].slot);
				}
			}

			N_TDC_HITS = n_tdc_hits;
			for(j=0;j<n_tdc_hits;j++)TDC[j]=tdc[j];
			FindXY_hits();

			/* Reset all histograms if SIGHUP signal is received */
			if(RESET_HISTOS){hreset(0," "); RESET_HISTOS=0;}

			/* Fill histograms */
			hist_fill(adc, tdc, n_tdc_hits);
		}
	}

	if(strlen(SESSION)){
		end_et();
	}else{
		CloseCODAFile(file);
	}
	hist_end();
	
	printf("\n\nEnd of file!\n\n");

	printf("Total events read    : %d  \n",NUM_EVENTS_READ);
	printf("Total physics events : %d\n\n",NUM_PHYSICS_EVENTS);

	return 0;
}


void ParseCommandLineArgs(int argc,char *argv[])
{
	int i;
	char *argptr;
	
	/* Print Usage statement if no arguments specified */
	if(argc==1)Usage();

	/* Loop through all command line arguments */
	for(i=1;i<argc;i++){
		argptr = argv[i];
		if(*argptr=='-'){
			argptr++;
			switch(*argptr){
				case 'S':
					strcpy(SESSION,++argptr);
					break;
				case 'o':
					strcpy(output_filename,++argptr);
					break;
				case 'N':
					NUM_EVENTS_TO_PROCESS=atoi(++argptr);
					break;
				case 'v':
					VERBOSE=1;
					break;
				case 'G':
					strcpy(GLOBAL_SECTION,++argptr);
					break;
				case 'h':
				default:
					Usage();
					break;

			}
		}else{
			strcpy(input_filename,argv[i]);
		}
	}

	/* Make sure an input file was specified */
	if(strlen(SESSION)==0 && strlen(input_filename)==0){
		fprintf(stderr,"No input file and no ET session specified!\n");
		Usage();
	}

	/* Determine output filename from input filename if none was not specified explicitly */
	if(strlen(output_filename)==0){
		if(strlen(SESSION)){
			strcpy(output_filename,"coda2hist.hbook");
		}else{
			strcpy(output_filename,input_filename);
			for(i=strlen(output_filename);i>0;i--)if(output_filename[i]=='.')break;
			if(i>0)
				strcpy(&output_filename[i],".hbook");
			else
				strcat(output_filename,".hbook");
		}
	}
}

void Usage(void)
{
	fprintf(stderr,"\nUsage:\n\tcoda2hist [options] -Ssession | filename\n\n");
	fprintf(stderr,"\toptions:\n\n");
	fprintf(stderr,"\t -Ssession      read events from ET session \"session\"\n");
	fprintf(stderr,"\t -oOutputfile   set output filename (default name is chosen)\n");
	fprintf(stderr,"\t                based on input filename)\n");
	fprintf(stderr,"\t -v             verbose (print LOTS of debugging information for \n");
	fprintf(stderr,"\t                every event\n");
	fprintf(stderr,"\t -Gname         Map pawc to global section name \n");
	fprintf(stderr,"\t -h             print this usage statement\n");
	fprintf(stderr,"\t -Nnum          only process \"num\" events\n");
	fprintf(stderr,"\n\n");
	exit(-1);
}




/* Book Histograms */
int hist_book(void)
{

	int istat;
	int i, bins;
	char title[100];
	int HID;

	/* open hbook file */
	if(strlen(output_filename) == 0) {
		sprintf(output_filename,"coda2hist.hbook");
	}
	fprintf(stderr, "Output file name is: %s\n", output_filename);
	quest_[9] = 65000;
	if(strlen(GLOBAL_SECTION)){
		printf("Mapping to global section \"%s\"\n",GLOBAL_SECTION);
		hlimap(MEMH,GLOBAL_SECTION);
	}else{
		hlimit(MEMH);
	}
	hropen(LUN, "lun", output_filename , "N", LREC, istat);

	/* book histos */
	for(i=1;i<=36;i++){
		sprintf(title,"Crystal %d",i);
		hbook1(100+i,title,8192,0.0,8192.0,0);

		sprintf(title,"Crystal %d - Crystal 1 + 100",i);
		hbook1(200+i,title,8192,0.0,8192.0,0);
	}

	hbook2(10,"TDC X channel vs. tdc", 1024, 0.0, 4096.0, 64, 0.5, 64.5, 0);
	hbook2(11,"TDC Y channel vs. tdc", 1024, 0.0, 4096.0, 64, 0.5, 64.5, 0);
	hbook2(12,"TDC Logic inputs vs. tdc", 1024, 0.0, 1024.0, 16, 80.5, 95.5, 0);

	hbook2(20,"X-Y coordinate detector", 64, 0.0, 64.0, 64, 0.0, 64.0, 0);
	hbook1(21,"X-Y TDC difference", 401, -200.0, 200.0, 0);
	hbook1(22,"X-Y TDC difference (time cut)", 401, -200.0, 200.0, 0);
	
}


/* Look for coorelated hits in the X-Y coordinate detectors */
void FindXY_hits(void)
{
	int i,j,k;

	/*---- Look for X-Y coincidences ----*/
	/* seperate X and Y hits */
	N_XHITS=N_YHITS=N_XYHITS=0;
	for(i=0;i<N_TDC_HITS;i++){

		/* X-Hits */
		if(TDC[i].slot==TDC_SLOT_1 && TDC[i].id<=63){
			if(N_XHITS<MAX_THITS){
				XHITS[N_XHITS++] = TDC[i];
			}else{
				printf("Too many X-hits. Aborting event.\n");
				break;
			}
		}

		/* Y-hits */
		if(TDC[i].slot==TDC_SLOT_2 && TDC[i].id<=63){
			if(N_YHITS<MAX_THITS){
				YHITS[N_YHITS++] = TDC[i];
			}else{
				printf("Too many Y-hits. Aborting event.\n");
				break;
			}
		}
	}
	/* X-Y-hits */
	for(j=0;j<N_XHITS;j++){
		for(k=0;k<N_YHITS;k++){
			if(fabs((float)XHITS[j].val-(float)YHITS[k].val)<=10){
				if(XHITS[j].val>730 && XHITS[j].val<770){
					if(N_XYHITS<MAX_THITS){
						XYHITS[N_XYHITS].x = XHITS[j];
						XYHITS[N_XYHITS++].y = YHITS[k];
					}else{
						printf("Too many X-Y-hits. Aborting event.\n");
						break;
					}
				}
			}
		}
	}
}


/* Fill Histograms */
void hist_fill(unsigned int adc[64], TDC1877_t *tdc, int n_tdc_hits)
{
	int i,j,k;
	int HID;
	float f,channel;

	for(i=1;i<=36;i++){
		if(adc[i-1]){
			hf1(100+i,(float)adc[i-1],1.0);
			hf1(200+i,(float)adc[i-1]-(float)adc[0]+100.0,1.0);
		}
	}

	for(i=0;i<n_tdc_hits;i++){
		if(tdc[i].edge != LRS1877_RISING_EDGE)continue;
		f 		   = (float)tdc[i].val;
		channel  = (float)tdc[i].id;
		HID		= tdc[i].slot==TDC_SLOT_1 ? 10:11;

		hf2(HID,f,channel,1.0);

		if(channel>64)hf2(12,f,channel,1.0);
	}

	for(i=0;i<N_XYHITS;i++){
		hf2(20,(float)XYHITS[i].x.id,(float)XYHITS[i].y.id,1.0);
	}

	/* X-Y time differnce */
	for(j=0;j<N_XHITS;j++){
		for(k=0;k<N_YHITS;k++){
			hf1(21,(float)XHITS[j].val-(float)YHITS[k].val,1.0);
			if((XHITS[j].val>730 && XHITS[j].val<770)
				|| (YHITS[j].val>730 && YHITS[j].val<770)){

				hf1(22,(float)XHITS[j].val-(float)YHITS[k].val,1.0);
			}
		}
	}
}

/* Close HBOOK file */
void hist_end(void)
{
	int i, icycle;
	int zero=0;


	hcdir("//lun"," ");
	hrout(0,1,"N");
	hrend("lun");

}


/* Read in the configuration files */
int ReadConfiguration(int run_number)
{
	int err;

	/*err = GetCalibConstants_Float(run_number, "pedestals",64,PEDESTAL);*/

	return err;
}

void ctrlCHandle(int x)
{
	fprintf(stderr,"Interrupt received. Quitting....\n");
	quit=1;

	/* If this is hit while waiting for the ET system to come back to life, */
	/* then the program will not continue until that happens. To give the   */
	/* the expected result, we force an exit here if reading from ET.       */
	if(strlen(SESSION)){
		end_et();
		sleep(1);
		fprintf(stderr,"Killing PID=%d\n",getpid());
		kill(getpid(),SIGKILL);
	}
}

void SIGHUPHandle(int x)
{
	fprintf(stderr,"SIGHUP signal received. Resetting histos ...\n");
	RESET_HISTOS=1;
}




