/*
//===============================================================================
//
// coda_parse.cc
// 
// 8/16/01  D. Lawrence
// 
// The routines in this file can be used to parse CODA formatted data.
// There are also routines which can parse the data within the CODA
// bank that is in the format presented by the digitization electronics.
// Specifically, the LRS1881M ADC and LRS1877 TDC.
// 
//===============================================================================
*/

#ifndef __CODA_PARSE_H__
#define __CODA_PARSE_H__


#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <ctype.h>

#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <signal.h>


#ifdef __cplusplus
// C++ specific
#include <stdiostream.h>
#include <libio.h>
#include <iostream.h>
#include <strstream.h>
#include <iomanip.h>
#include <pthread.h>
#endif /* __cplusplus */

#define CODA_EVENT_TYPE_PHYSICS	0x10	// CODA_header_t->bank.data_type
#define CODA_EVENT_TYPE_SYNC		16		// CODA_bank_t->tag
#define CODA_EVENT_TYPE_PRESTART	17		//       .
#define CODA_EVENT_TYPE_GO			18		//       .
#define CODA_EVENT_TYPE_PAUSE		19		//       .
#define CODA_EVENT_TYPE_END		20		//       .

#define LRS1877_RISING_EDGE  0
#define LRS1877_FALLING_EDGE 1


typedef struct {
	unsigned int length;
	unsigned char num,data_type;
	unsigned short tag;
}CODA_bank_t;


typedef struct {
	CODA_bank_t bank;
	int data_bank;		/* The first data bank */
}CODA_header_t;


typedef struct {
	CODA_bank_t bank;
	unsigned int event_number;
	unsigned int event_class;
	unsigned int status;
	int next_bank;
}CODA_event_id_t;


typedef struct {
	unsigned int length;
	unsigned char counter,type,roc,status;
	unsigned int data;
}CODA_roc_data_t;


typedef struct {
	unsigned int slot;
	unsigned int id;
	unsigned int edge;
	unsigned int val;
}TDC1877_t;


/* Routine declarations (make C compatible) */
#ifdef __cplusplus
extern "C" {
#endif
int GetBankPtrs(char *data, CODA_event_id_t **event_id, CODA_roc_data_t **roc_data, int *Nptrs );
int ParseLRS1881Mdata(CODA_roc_data_t *roc_data,int slot,unsigned int adc[64]);
int ParseLRS1877data(CODA_roc_data_t *roc_data,int slot,TDC1877_t *tdc, int *Nhits);
int ParseAlldata(CODA_roc_data_t *roc_data
	,unsigned int LRS1881M_slots, unsigned int adc[64]
	,unsigned int LRS1877_slots, TDC1877_t *tdc, int *Nhits);
#ifdef __cplusplus
}
#endif

#endif /*__CODA_PARSE_H__*/





