#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <thread.h>
#include <et.h>
 
#define NUMEVENTS 500000
#define CHUNK 100
 
main(int argc,char **argv)
{ 
	int i, j, status, swtch, numread, totalread=0;
	int con[ET_STATION_SELECT_INTS];
	et_statconfig sconfig;
	et_openconfig openconfig;
	et_event   *pe[CHUNK];
	et_att_id  attach1;
	et_stat_id my_stat;
	et_sys_id  id;
	int selections[] = {17,15,-1,-1}; /* 17 & 5 are arbitrary */

	if(argc != 4) {
	  printf("Usage: et_client <et_filename> <station_name> <mode>\n");
	  exit(1);
	}

	/* open local ET system and don't wait for it */
	et_open_config_init(&openconfig);
	if (et_open(&id, argv[1], openconfig) != ET_OK) {
	  printf("et_client: et_open problems\n");
	  exit(1);
	}
	et_open_config_destroy(openconfig);

	/* User selects which type/mode of station to create - got 6 choices.
	 * (Of course, many more combinations of settings are possible.)
	 */
	swtch = atoi(argv[3]);

	/* set some common values */
	et_station_config_init(&sconfig);
	et_station_config_setuser(sconfig, ET_STATION_USER_MULTI);
	et_station_config_setrestore(sconfig, ET_STATION_RESTORE_OUT);
	et_station_config_setprescale(sconfig, 5);
	et_station_config_setcue(sconfig, 20);
	 
	if (swtch == 1) {
	  /* DD system "all" mode */
	  et_station_config_setselect(sconfig, ET_STATION_SELECT_ALL);
	  et_station_config_setblock(sconfig, ET_STATION_BLOCKING);
	}
	else if (swtch == 2) {
	  /* DD system "on req" mode */
	  et_station_config_setselect(sconfig, ET_STATION_SELECT_ALL);
	  et_station_config_setblock(sconfig, ET_STATION_NONBLOCKING);
	}
	else if (swtch == 3) {
	  /* DD system "condition" mode */
	  et_station_config_setselect(sconfig, ET_STATION_SELECT_MATCH);
	  et_station_config_setblock(sconfig, ET_STATION_BLOCKING);
	  et_station_config_setselectwords(sconfig, selections);
	}
	else if (swtch == 4) {
	  /* new non-blocking "condition" mode */
	  et_station_config_setselect(sconfig, ET_STATION_SELECT_MATCH);
	  et_station_config_setblock(sconfig, ET_STATION_NONBLOCKING);
	  et_station_config_setselectwords(sconfig, selections);
	}
	else if (swtch == 5) {
	  /* user's condition, blocking mode */
	  et_station_config_setselect(sconfig, ET_STATION_SELECT_USER);
	  et_station_config_setblock(sconfig, ET_STATION_BLOCKING);
	  et_station_config_setselectwords(sconfig, selections);
	  if (et_station_config_setfunction(sconfig, "et_my_function") == ET_ERROR) {
	    printf("et_client: cannot set function\n");
	    exit(1);
	  }
	  if (et_station_config_setlib(sconfig, "/.../libet_user.so") == ET_ERROR) {
	    printf("et_client: cannot set library\n");
	    exit(1);
	  }
	}
	else if (swtch == 6) {
	  /* user's condition, nonblocking mode */
	  et_station_config_setselect(sconfig, ET_STATION_SELECT_USER);
	  et_station_config_setblock(sconfig, ET_STATION_NONBLOCKING);
	  et_station_config_setselectwords(sconfig, selections);
	  if (et_station_config_setfunction(sconfig, "et_my_function") == ET_ERROR) {
	    printf("et_client: cannot set function\n");
	    exit(1);
	  }
	  if (et_station_config_setlib(sconfig, "/.../libet_user.so") == ET_ERROR) {
	    printf("et_client: cannot set library\n");
	    exit(1);
	  }
	}

	/* set level of debug output */
	et_system_setdebug(id, ET_DEBUG_INFO);

	/* create the station */
	if ((status = et_station_create(id, &my_stat, argv[2], sconfig)) < ET_OK) {
	  if (status == ET_ERROR_EXISTS) {
	    /* my_stat contains pointer to existing station */;
	    printf("et_client: station already exists\n");
	  }
	  else if (status == ET_ERROR_TOOMANY) {
	    printf("et_client: too many stations created\n");
	    goto error;
	  }
	  else {
	    printf("et_client: error in station creation\n");
	    goto error;
	  }
	}
	et_station_config_destroy(sconfig);

	/* attach to the newly created station */
	if (et_station_attach(id, my_stat, &attach1) < 0) {
	  printf("et_client: error in station attach\n");
	  goto error;
	}

	/* loop, while ET system is alive, to read and write events */
	while (et_alive(id)) {
	  /* example of reading array of up to "CHUNK" events */
	  status = et_events_get(id, attach1, pe, ET_SLEEP, NULL, CHUNK, &numread);
	  if (status == ET_OK) {
	    /* everything is OK*/;
	  }
	  else if (status == ET_ERROR_DEAD) {
	    printf("et_client: ET is dead\n");
	    goto end;
	  }
	  else if (status == ET_ERROR_TIMEOUT) {
	    printf("et_client: got timeout\n");
	    goto end;
	  }
	  else if (status == ET_ERROR_EMPTY) {
	    printf("et_client: no events\n");
	    goto end;
	  }
	  else if (status == ET_ERROR_BUSY) {
	    printf("et_client: station is busy\n");
	    goto end;
	  }
	  else if (status == ET_ERROR_WAKEUP) {
	    printf("et_client: someone told me to wake up\n");
	    goto end;
	  }
	  else if (status != ET_OK) {
	    printf("et_client: get error\n");
	    goto error;
	  }
	 
	  /* print data */
	  if (0) {
	    int pri, len, *data;
	    for (j=0; j < numread; j++) {
	      et_event_getdata(pe[j], (void **) &data);
	      et_event_getpriority(pe[j], &pri);
	      et_event_getlength(pe[j], &len);
	      et_event_getcontrol(pe[j], con);
	      printf("et_client data = %d, pri = %d, len = %d\n", *data, pri, len); 
	      for (i=0; i < ET_STATION_SELECT_INTS; i++) {
	        printf(" con[%d] = %d\n", i, con[i]);
	      }
	    }
	  }

	  /* example of putting array of events */
	  status = et_events_put(id, attach1, pe, numread);
	  if (status == ET_ERROR_DEAD) {
	    printf("et_client: ET is dead\n");
	    goto end;
	  }
	  else if (status != ET_OK) {
	    printf("et_client: put error\n");
	    goto error;
	  }
	  totalread += numread;

	end:
	  /* print something out after having read NUMEVENTS events */
	  if (totalread >= NUMEVENTS) {
	    totalread = 0;
	    printf(" et_client: %d events\n", NUMEVENTS);
	  }

	  /* if ET system is dead, wait here until it comes back */
	  if (!et_alive(id)) {
	    et_wait_for_alive(id);
	  }
	} /* while(alive) */

	error:
	free(pe);
	printf("et_client: ERROR\n");
	exit(0);
}
